@@ -16,20 +16,20 @@ CGI::Buffer - Verify, Cache and Optimise CGI Output
 
 =head1 VERSION
 
-Version 0.77
+Version 0.78
 
 =cut
 
-our $VERSION = '0.77';
+our $VERSION = '0.78';
 
 =head1 SYNOPSIS
 
 CGI::Buffer verifies the HTML that you produce by passing it through
 C<HTML::Lint>.
 
-CGI::Buffer optimises CGI programs by compressing output to speed up
-the transmission and by nearly seamlessly making use of client and
-server caches.
+CGI::Buffer optimises FCGI programs by reducing, filtering and compressing
+output to speed up the transmission and by nearly seamlessly making use of
+client and server caches.
 
 To make use of client caches, that is to say to reduce needless calls
 to your server asking for the same data, all you need to do is to
@@ -114,9 +114,8 @@ END {
 		if($ENV{'HTTP_IF_MODIFIED_SINCE'}) {
 			$logger->debug("HTTP_IF_MODIFIED_SINCE: $ENV{HTTP_IF_MODIFIED_SINCE}");
 		}
-		$logger->debug("Generate_etag = $generate_etag");
-		$logger->debug("Generate_304 = $generate_304");
-		$logger->debug("Generate_last_modified = $generate_last_modified");
+		$logger->debug("Generate_etag = $generate_etag", "Generate_304 = $generate_304",
+			"Generate_last_modified = $generate_last_modified");
 
 		# This will cause everything to get flushed and prevent
 		# outputs to the logger.  We need to do that now since
@@ -557,14 +556,7 @@ END {
 		# CGI::Buffer
 		unshift @o, split(/\r\n/, $headers);
 		if($body && $send_body) {
-			my $already_done = 0;
-			foreach(@o) {
-				if(/^Content-Length: /) {
-					$already_done = 1;
-					last;
-				}
-			}
-			unless($already_done) {
+			unless(grep(/^Content-Length: /, @o)) {
 				push @o, "Content-Length: $body_length";
 			}
 		}
@@ -661,6 +653,7 @@ sub _check_modified_since {
 	}
 }
 
+# Reduce output, e.g. remove superfluous white-space.
 sub _optimise_content {
 	# FIXME: regex bad, HTML parser good
 	# Regexp::List - wow!
@@ -781,7 +774,7 @@ Set various options and override default values.
 	cache_key => 'string',	# key for the cache
 	cache_age => '10 minutes',	# how long to store responses in the cache
 	logger => $logger,
-	lint->content => 0,	# Pass through HTML::Lint
+	lint_content => 0,	# Pass through HTML::Lint
 	generate_304 => 1,	# Generate 304: Not modified
     );
 
@@ -849,7 +842,8 @@ sub init {
 	my $pos = $CGI::Buffer::buf->getpos;
 	if($pos > 0) {
 		if(defined($logger)) {
-			$logger->warn("Too late to call init, $pos characters have been printed");
+			my @call_details = caller(0);
+			$logger->warn("Too late to call init, $pos characters have been printed, caller line $call_details[2] of $call_details[1]");
 		} else {
 			# Must do Carp::carp instead of carp for Test::Carp
 			Carp::carp "Too late to call init, $pos characters have been printed";
@@ -1062,19 +1056,21 @@ sub _my_age {
 	return $script_mtime;
 }
 
-sub _should_gzip {
-	if($compress_content && $send_body && ($ENV{'HTTP_ACCEPT_ENCODING'} || $ENV{'HTTP_TE'})) {
+sub _should_gzip
+{
+	if(defined($send_body) && ($send_body == 0)) {
+		return '';
+	}
+	if($compress_content && ($ENV{'HTTP_ACCEPT_ENCODING'} || $ENV{'HTTP_TE'})) {
+		if(scalar(@content_type)) {
+			if($content_type[0] ne 'text') {
+				return '';
+			}
+		}
 		my $accept = lc($ENV{'HTTP_ACCEPT_ENCODING'} ? $ENV{'HTTP_ACCEPT_ENCODING'} : $ENV{'HTTP_TE'});
-		foreach my $encoding ('x-gzip', 'gzip') {
-			$_ = $accept;
-			if(@content_type && $content_type[0]) {
-				if (m/$encoding/i && (lc($content_type[0]) eq 'text')) {
-					return $encoding;
-				}
-			} else {
-				if (m/$encoding/i) {
-					return $encoding;
-				}
+		foreach my $method(split(/,\s?/, $accept)) {
+			if(($method eq 'gzip') || ($method eq 'x-gzip') || ($method eq 'br')) {
+				return $method;
 			}
 		}
 	}
@@ -1104,19 +1100,36 @@ sub _compress {
 		return;
 	}
 
-	require Compress::Zlib;
-	Compress::Zlib->import;
+	if($encoding eq 'gzip') {
+		require Compress::Zlib;
+		Compress::Zlib->import;
 
-	# Avoid 'Wide character in memGzip'
-	unless($encode_loaded) {
-		require Encode;
-		$encode_loaded = 1;
-	}
-	my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($body));
-	if(length($nbody) < length($body)) {
-		$body = $nbody;
-		push @o, "Content-Encoding: $encoding";
-		push @o, "Vary: Accept-Encoding";
+		# Avoid 'Wide character in memGzip'
+		unless($encode_loaded) {
+			require Encode;
+			$encode_loaded = 1;
+		}
+		my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($body));
+		if(length($nbody) < length($body)) {
+			$body = $nbody;
+			push @o, "Content-Encoding: $encoding";
+			push @o, "Vary: Accept-Encoding";
+		}
+	} elsif($encoding eq 'br') {
+		require IO::Compress::Brotli;
+		IO::Compress::Brotli->import();
+
+		# Avoid 'Wide character in memGzip'
+		unless($encode_loaded) {
+			require Encode;
+			$encode_loaded = 1;
+		}
+		my $nbody = bro(Encode::encode_utf8($body));
+		if(length($nbody) < length($body)) {
+			$body = $nbody;
+			push @o, "Content-Encoding: $encoding";
+			push @o, "Vary: Accept-Encoding";
+		}
 	}
 }
 
@@ -1204,7 +1217,7 @@ L<http://search.cpan.org/dist/CGI-Buffer/>
 
 =head1 ACKNOWLEDGEMENTS
 
-The inspiration and code for some if this is cgi_buffer by Mark
+The inspiration and code for some of this is cgi_buffer by Mark
 Nottingham: http://www.mnot.net/cgi_buffer.
 
 =head1 LICENSE AND COPYRIGHT
@@ -1218,7 +1231,7 @@ The licence for cgi_buffer is:
 
     This software is provided 'as is' without warranty of any kind."
 
-The rest of the program is Copyright 2011-2015 Nigel Horne,
+The rest of the program is Copyright 2011-2016 Nigel Horne,
 and is released under the following licence: GPL
 
 =cut
